@@ -12,11 +12,11 @@ Schema versioning for DBIx::Class with version information embedded inline in th
 
 =head1 VERSION
 
-Version 0.002
+Version 0.003
 
 =cut
 
-our $VERSION = '0.002';
+our $VERSION = '0.003';
 
 =head1 SYNOPSIS
 
@@ -255,7 +255,6 @@ use strict;
 use base 'DBIx::Class::Schema::Versioned';
 
 use Carp;
-use Data::Dumper::Concise;
 use SQL::Translator;
 use SQL::Translator::Diff;
 use Try::Tiny;
@@ -376,9 +375,10 @@ sub upgrade_single_step {
 
     # add Upgrade versions
     my $upgradeclass = ref($self) . "::Upgrade";
+    my @before_upgrade_subs;
     eval {
         eval "require $upgradeclass" or return;
-        my @sql = $upgradeclass->upgrade_to($target_version);
+        @before_upgrade_subs = $upgradeclass->before_upgrade($target_version);
     };
 
     # translate current schema
@@ -443,11 +443,14 @@ sub upgrade_single_step {
         }
     )->compute_differences->produce_diff_sql;
 
-    my $exception;
 
+    my $exception;
     try {
         $self->txn_do(
             sub {
+                foreach my $sub (@before_upgrade_subs) {
+                    $sub->($self) or die;
+                }
                 foreach my $line (@diff) {
 
                     # drop comments and BEGIN/COMMIT
@@ -455,6 +458,7 @@ sub upgrade_single_step {
                     $self->storage->dbh_do(
                         sub {
                             my ( $storage, $dbh ) = @_;
+                            #print STDERR $line;
                             $dbh->do($line);
                         }
                     );
@@ -466,6 +470,13 @@ sub upgrade_single_step {
         $exception = $_;
     };
 
+#        my $rset = $self->resultset('Foo')->search({});
+#        if ($rset) {
+#            while ( my $res = $rset->next ) {
+#                print STDERR "XX " . $res->width . "\n";
+#            }
+#        }
+
     if ( defined $exception ) {
         $self->throw_exception($exception);
     }
@@ -508,6 +519,7 @@ sub versioned_schema {
             my $until   = $versioned->{until};
             my $since   = $versioned->{since};
             my $changes = $versioned->{changes};
+            my $renamed = $versioned->{renamed_from};
 
             # handle since/until first
 
@@ -519,6 +531,23 @@ sub versioned_schema {
             $self->_since_until( $pversion, $since, $until, $name, $sub,
                 $source );
 
+            # handled renamed column
+
+            if ( $since && $renamed && $_version eq $since) {
+
+                # we need renamed_from to be in "extra" for SQLT
+
+                $column_info->{extra}->{renamed_from} = $renamed;
+
+                unless ( $source->remove_column($column)
+                    && $source->add_column( $column => $column_info ) )
+                {
+                    $self->throw_exception(
+                        "Failed to apply renamed_from for $name" );
+                }
+
+            }
+
             # handle changes
 
             if ($changes) {
